macro_rules! command_impl {
    (
        $command:ident<$tconfig:ty>;
        $slf:ident => $init:block
    ) => {
        command_impl! { $command<$tconfig>;
        $slf, ctx => $init }
    };
    (
        $command:ident<$tconfig:ty>;
        $slf:ident, $ctx:ident => $init:block
    ) => {
        command_impl! { $command<$tconfig,()>;
        $slf, $ctx => $init; { $crate::command::CommandResult::Done }; {} }
    };
    (
        $command:ident<$tconfig:ty,$tstate:ty>;
        $slf:ident, $ctx:ident => $init:block;
        $step:block;
        $finish:block
    ) => {
        impl $crate::command::CommandImp for $tconfig {
            type State = $tstate;

            fn init(&$slf, $ctx: &mut $crate::command::CommandContext<Self>) {
                use $crate::command::GetCommandInfo;
                debug!(
                    "{}({}) init",
                    stringify!($command),
                    $ctx.name()
                );
                $init;
            }

            fn step(&$slf, $ctx: &mut $crate::command::CommandContext<Self>) -> $crate::command::CommandResult {
                use $crate::command::GetCommandInfo;
                debug!(
                    "{}({}) step",
                    stringify!($command),
                    $ctx.name()
                );
                $step
            }

            fn finish(&$slf, $ctx: &mut $crate::command::CommandContext<Self>) {
                use $crate::command::GetCommandInfo;
                debug!(
                    "{}({}) finish({:?})",
                    stringify!($command),
                    $ctx.name(),
                    $ctx.status(),
                );
                $finish;
            }
        }
        pub type $command = $crate::command::GenericCommand<$tconfig>;
    };
    (
        $command:ident<$tconfig:ty,$tstate:ty>;
        $slf:ident => $init:block;
        $step:block;
        $finish:block
    ) => {
        command_impl! { $command<$tconfig,$tstate>;
        $init_self, cts => $init; $step; $finish }
    };
}
