use std::sync::Arc;
use std::cell::UnsafeCell;

pub trait MotorController {
    fn id(&self) -> u16;
    fn set(&self, value: f64);
    fn get(&self) -> f64;
}

pub trait MotorControllerId: MotorController {
    fn new(id: u16) -> Self;
}

pub type MotorControllerRef = Arc<dyn MotorController>;

pub trait MotorControllerNewRef {
    fn new_ref(id: u16) -> MotorControllerRef;
}

impl<T: MotorControllerId + 'static> MotorControllerNewRef for T {
    fn new_ref(id: u16) -> MotorControllerRef {
        Arc::new(T::new(id))
    }
}

pub struct MockMotorController {
    id: u16,
    value: UnsafeCell<f64>, // meh
}

impl MotorControllerId for MockMotorController {
    fn new(id: u16) -> Self {
        MockMotorController { id, value: UnsafeCell::new(0.0) }
    }
}

impl MotorController for MockMotorController {
    fn id(&self) -> u16 {
        self.id
    }

    fn set(&self, value: f64) {
        unsafe { *self.value.get() = value; }
        debug!("MotorController id {} value={}", self.id, value);
    }

    fn get(&self) -> f64 {
        unsafe { *self.value.get() }
    }
}
